/*
 * Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.  
 *
 * * Neither the name of the University of California, Berkeley nor
 *   the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior 
 *   written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package blog.distrib;

/**
 * A distribution over {0,1}. It takes one parameter <code>p</code>, which is
 * the probability
 * of <code>true</code>.
 * 
 * The actual implementation relies on BooleanDistrib.
 * 
 * @see blog.distrib.BooleanDistrib.
 * 
 * @author cgioia
 * @since June 17, 2014
 */
public class Bernoulli implements CondProbDistrib {

  public Bernoulli() {
    booldist = new BooleanDistrib();
  }

  /*
   * (non-Javadoc)
   * 
   * @see blog.distrib.CondProbDistrib#getProb(java.lang.Object)
   */
  @Override
  public double getProb(Object value) {
    if (value.equals(1)) {
      return booldist.getProb(true);
    } else if (value.equals(0)) {
      return booldist.getProb(false);
    }
    return 0.0;
  }

  /*
   * (non-Javadoc)
   * 
   * @see blog.distrib.CondProbDistrib#getLogProb(java.lang.Object)
   */
  @Override
  public double getLogProb(Object value) {
    if (value.equals(1)) {
      return booldist.getLogProb(true);
    } else if (value.equals(0)) {
      return booldist.getLogProb(false);
    }
    return Double.NEGATIVE_INFINITY;
  }

  /*
   * (non-Javadoc)
   * 
   * @see blog.distrib.CondProbDistrib#sampleVal()
   */
  @Override
  public Object sampleVal() {
    return ((Boolean) booldist.sampleVal()) ? 1 : 0;
  }

  /**
   * set parameters for Bernoulli distribution
   * 
   * @param params
   *          An array of a single double
   *          params[0]: <code>p</code>
   * 
   * @see blog.distrib.CondProbDistrib#setParams(java.lang.Object[])
   */
  @Override
  public void setParams(Object[] params) {
    booldist.setParams(params);
  }

  @Override
  public String toString() {
    return getClass().getName();
  }

  @Override
  public Object[] getFiniteSupport() {
    return finiteSupport;
  }

  /**
   * Actual holder of underlying boolean distribution.
   */
  private BooleanDistrib booldist;
  private static final Object[] finiteSupport = { 0, 1 };
}
